"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.RspackCLI = void 0;
const helpers_1 = require("yargs/helpers");
const yargs_1 = __importDefault(require("yargs"));
const util_1 = __importDefault(require("util"));
const path_1 = __importDefault(require("path"));
const fs_1 = __importDefault(require("fs"));
const build_1 = require("./commands/build");
const serve_1 = require("./commands/serve");
const core_1 = require("@rspack/core");
const defaultConfig = "rspack.config.js";
const defaultEntry = "src/index.js";
class RspackCLI {
    constructor() {
        this.colors = this.createColors();
        this.program = (0, yargs_1.default)();
    }
    async createCompiler(options) {
        let config = await this.loadConfig(options);
        config = await this.buildConfig(config, options);
        const compiler = (0, core_1.createCompiler)(config);
        return compiler;
    }
    createColors(useColor) {
        const { createColors, isColorSupported } = require("colorette");
        let shouldUseColor;
        if (useColor) {
            shouldUseColor = useColor;
        }
        else {
            shouldUseColor = isColorSupported;
        }
        return {
            ...createColors({ useColor: shouldUseColor }),
            isColorSupported: shouldUseColor
        };
    }
    getLogger() {
        return {
            error: val => console.error(`[rspack-cli] ${this.colors.red(util_1.default.format(val))}`),
            warn: val => console.warn(`[rspack-cli] ${this.colors.yellow(val)}`),
            info: val => console.info(`[rspack-cli] ${this.colors.cyan(val)}`),
            success: val => console.log(`[rspack-cli] ${this.colors.green(val)}`),
            log: val => console.log(`[rspack-cli] ${val}`),
            raw: val => console.log(val)
        };
    }
    async run(argv) {
        this.program.usage("[options]");
        this.program.scriptName("rspack");
        this.registerCommands();
        await this.program.parseAsync((0, helpers_1.hideBin)(argv));
    }
    async registerCommands() {
        const builtinCommands = [new build_1.BuildCommand(), new serve_1.ServeCommand()];
        for (const command of builtinCommands) {
            command.apply(this);
        }
    }
    async buildConfig(item, options) {
        if (options.mode) {
            item.mode = options.mode;
        }
        if (!item.mode &&
            process.env &&
            process.env.NODE_ENV &&
            (process.env.NODE_ENV === "development" ||
                process.env.NODE_ENV === "production" ||
                process.env.NODE_ENV === "none")) {
            item.mode = process.env.NODE_ENV;
        }
        if (!item.mode) {
            item.mode = "production";
        }
        console.log("mode:", options.mode, item.mode);
        return item;
    }
    async loadConfig(options) {
        let loadedConfig;
        // if we pass config paras
        if (options.config) {
            const resolvedConfigPath = path_1.default.resolve(process.cwd(), options.config);
            if (!fs_1.default.existsSync(resolvedConfigPath)) {
                throw new Error(`config file "${resolvedConfigPath}" not exists`);
            }
            loadedConfig = require(resolvedConfigPath);
        }
        else {
            let defaultConfigPath = path_1.default.resolve(process.cwd(), defaultConfig);
            if (fs_1.default.existsSync(defaultConfigPath)) {
                loadedConfig = require(defaultConfigPath);
            }
            else {
                let entry = {};
                if (options.entry) {
                    console.log("entry:", options.entry);
                    entry = {
                        main: options.entry.map(x => path_1.default.resolve(process.cwd(), x))[0] // Fix me when entry supports array
                    };
                }
                else {
                    entry = {
                        main: path_1.default.resolve(process.cwd(), defaultEntry)
                    };
                }
                loadedConfig = {
                    entry
                };
            }
        }
        return loadedConfig;
    }
}
exports.RspackCLI = RspackCLI;
//# sourceMappingURL=rspack-cli.js.map